package fr.ippon.kafkaLatency.streaming;


import com.fasterxml.jackson.databind.ObjectMapper;
import fr.ippon.kafkaLatency.model.Person;
import fr.ippon.kafkaLatency.utils.Config;
import fr.ippon.kafkaLatency.utils.UtilStats;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.serialization.Serdes;
import org.apache.kafka.streams.KafkaStreams;
import org.apache.kafka.streams.StreamsConfig;
import org.apache.kafka.streams.kstream.KStreamBuilder;
import org.apache.kafka.streams.kstream.Predicate;
import org.apache.kafka.streams.processor.TimestampExtractor;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Properties;
import java.util.TimeZone;

/**
 * Ippon Tech 2018.
 */
public class KafkaStreaming  {
    static ObjectMapper mapper = new ObjectMapper();

    private final static Logger logger = Logger.getLogger(KafkaStreaming.class);

    public static class MyTimestampExtractor implements TimestampExtractor {

        UtilStats stats = new UtilStats();

        public MyTimestampExtractor() {
            super();
        }

        @Override
        public long extract(ConsumerRecord<Object, Object> consumerRecord, long t) {

            LocalDateTime parsedDate = LocalDateTime.ofInstant(Instant.ofEpochMilli(consumerRecord.timestamp()), TimeZone
                    .getDefault().toZoneId());

            LocalDateTime now = LocalDateTime.now();

            long millis = ChronoUnit.MILLIS.between(parsedDate, now);

            stats.treatStats(millis);

            return consumerRecord.timestamp();
        }
    }

    public static void main(String[] args) {
        try {


        String brokers = Config.getProperty("bootstrap.servers");
        String topic = Config.getProperty("topic");
        String numPartitions = Config.getProperty("numPartitions");

        Properties props = new Properties();

        props.put(StreamsConfig.APPLICATION_ID_CONFIG, "kafka stream");
        props.put(StreamsConfig.BOOTSTRAP_SERVERS_CONFIG, brokers);
        props.put(StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG, Serdes.String().getClass().getName());
        props.put(StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG, Serdes.String().getClass().getName());
        props.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, "latest");
        props.put(StreamsConfig.DEFAULT_TIMESTAMP_EXTRACTOR_CLASS_CONFIG, MyTimestampExtractor.class);
        props.put(StreamsConfig.NUM_STREAM_THREADS_CONFIG, numPartitions);

        KStreamBuilder builder = new KStreamBuilder();

        builder.stream(Serdes.String(), Serdes.String(), topic).filter(new Predicate<String, String>() {
            @Override
            public boolean test(String key, String value) {
                try {

                    mapper.readValue(value, Person.class);

                } catch (IOException e) {
                    logger.error("Error " + e.getMessage());
                }
                return false;
            }
        }).print();


        KafkaStreams streams = new KafkaStreams(builder, props);

        streams.start();

        } catch (Exception e) {
            logger.error("Error " + e.getMessage());
        }

    }


}
