package fr.ippon.kafkaLatency.utils;

import com.fasterxml.jackson.databind.JsonNode;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.common.Metric;
import org.apache.kafka.common.MetricName;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;

/**
 * inspired from http://cloudurable.com/blog/kafka-tutorial-kafka-producer-advanced-java-examples/index.html
 */
public class MetricsProducerReporter {
    private final Producer<String, JsonNode> producer;
    private final Logger logger =
            LoggerFactory.getLogger(MetricsProducerReporter.class);

    public MetricsProducerReporter(
            final Producer<String, JsonNode> producer) {
        this.producer = producer;
    }


    static class MetricPair {
        private final MetricName metricName;
        private final Metric metric;

        MetricPair(MetricName metricName, Metric metric) {
            this.metricName = metricName;
            this.metric = metric;
        }

        public String toString() {
            return metricName.group() + "." + metricName.name();
        }
    }

    public void displayMetrics() {
        final Map<MetricName, ? extends Metric> metrics
                = producer.metrics();

        final Map<String, MetricPair> metricsDisplayMap = metrics.entrySet().stream()
                .map(entry -> new MetricPair(entry.getKey(), entry.getValue()))
                .collect(Collectors.toMap(
                        MetricPair::toString, it -> it, (a, b) -> a, TreeMap::new
                ));


        //Output metrics
        final StringBuilder builder = new StringBuilder(255);
        builder.append("\n---------------------------------------\n");


        metricsDisplayMap.forEach((name, metricPair) -> builder.append(String.format(Locale.FRANCE, "%45s\t%25s\t%,-14.2f\t%s\n",
                name,
                metricPair.metricName.name(),
                metricPair.metric.value(),
                metricPair.metricName.description()
        )));
        builder.append("\n---------------------------------------\n");
        logger.info(builder.toString());
    }


}
